from enum import Enum

from wedeliver_core.helpers.service_config import ServiceConfig


class Service:
    CAPTAIN = ServiceConfig('CAPTAIN_SERVICE')
    FINANCE = ServiceConfig('FINANCE_SERVICE')
    SDD = ServiceConfig('SDD_SERVICE')
    SUPPLIER = ServiceConfig('SUPPLIER_SERVICE')
    PN = ServiceConfig('PN_SERVICE')
    FINTECH = ServiceConfig('FINTECH_SERVICE')
    STC = ServiceConfig('STC_SERVICE')
    AUTH = ServiceConfig('AUTH_SERVICE')
    MAIL = ServiceConfig('MAIL_SERVICE')
    SMS = ServiceConfig('SMS_SERVICE')
    APILAYER = ServiceConfig('APILAYER_SERVICE')
    INVOICE = ServiceConfig('INVOICE_SERVICE')
    ADDRESS = ServiceConfig('ADDRESS_SERVICE')
    PUBLIC = ServiceConfig('PUBLIC_SERVICE')
    INTERNAL_NOTIFICATION = ServiceConfig('INTERNAL_NOTIFICATION_SERVICE')


class QueryTypes(Enum):
    SIMPLE_TABLE = 1
    FUNCTION = 2
    SEARCH = 3



from wedeliver_core.helpers.sql import sql


def search_function(table_name, search_list):
    """
    """
    validation_error = []

    def _find_matched(db_results, key, search_item):
        for db_instance in db_results:
            if db_instance.get(key) == search_item.get("search_value"):
                return db_instance.get("id")
        return None

    for item_dict in search_list:
        # query = db.session.query(model)

        input_group_search_key = item_dict.get('search_key') or "id"

        input_group = item_dict.get('inputs')
        input_group_list = [obj.get("search_value") for obj in input_group]

        # model_key = model.__dict__.get(input_group_search_key)
        # result_db = query.filter(model_key.in_(input_group_list)).all()
        # result_db_list = [obj.__dict__.get(input_group_search_key) for obj in result_db]

        query = """
                SELECT *
                FROM {table_name}
                WHERE {input_group_search_key} IN ('{input_group_list}')
                """.format(
            input_group_list=', '.join(str(value) for value in input_group_list),
            table_name=table_name,
            input_group_search_key=input_group_search_key
        )
        result_db = sql(query)
        result_db_list = [obj.get(input_group_search_key) for obj in result_db]
        # if len(input_group_list) != len(result_db_list):
        for obj in input_group:
            if obj.get("search_value") not in result_db_list:
                validation_error.append(
                    "{} '{}' is not exists".format(input_group_search_key, obj.get("search_value")))
            elif result_db_list.count(obj.get("search_value")) > 1:
                validation_error.append(
                    "{} '{}' too many exists, can not determine witch one".format(input_group_search_key,
                                                                                  obj.get("search_value")))
            else:
                obj.update(dict(
                    matched_id=_find_matched(db_results=result_db, key=input_group_search_key, search_item=obj)
                ))

    return search_list, validation_error



from .handle_exceptions import handle_exceptions
from .serializer import serializer
from .handle_response import handle_response
from .handle_auth import handle_auth

__all__ = ["handle_exceptions", "serializer", "handle_response", "handle_auth"]



from enum import Enum


class Topics(Enum):
    SEND_MAIL = 'send_mail'
    SEND_SMS = 'send_sms'
    SEND_PUSH_NOTIFICATION = 'send_push'
    INTERNAL_NOTIFICATION_MESSAGE = 'internal_notification_message'
    CREATE_STC_SUPPLIER_PAYMENT_TRANSACTION = 'create_stc_supplier_payment_transaction'
